@@ -23,7 +23,7 @@
 
 #include <glib.h>
 #include <epan/guid-utils.h>
-#include <epan/wmem/wmem.h>
+#include <epan/wmem_scopes.h>
 #include <epan/ipv6.h>
 
 #include <wsutil/nstime.h>
@@ -74,7 +74,6 @@ typedef struct tvbuff tvbuff_t;
  *    This (obviously) also applies to any tvbs chained to the tvb handed
  *    to the dissector.
  *  - Can create its own tvb chain (using tvb_new_real_data() which the
- *
  *    dissector is free to manage as desired.
  * @{
  */
@@ -105,10 +104,17 @@ typedef void (*tvbuff_free_cb_t)(void*);
 /** Extracts 'number of bits' starting at 'bit offset'.
  * Returns a pointer to a newly initialized g_malloc'd REAL_DATA
  * tvbuff with the bits octet aligned.
+ * Bits are counted from MSB (0) to LSB (7) within octets.
  */
 WS_DLL_PUBLIC tvbuff_t *tvb_new_octet_aligned(tvbuff_t *tvb,
     guint32 bit_offset, gint32 no_of_bits);
 
+/** Extracts 'number of bits' starting at 'bit offset'.
+ * Bits are counted from LSB (0) to MSB (7) within octets.
+ */
+WS_DLL_PUBLIC tvbuff_t *tvb_new_octet_right_aligned(tvbuff_t *tvb,
+    guint32 bit_offset, gint32 no_of_bits);
+
 WS_DLL_PUBLIC tvbuff_t *tvb_new_chain(tvbuff_t *parent, tvbuff_t *backing);
 
 WS_DLL_PUBLIC tvbuff_t *tvb_clone(tvbuff_t *tvb);
@@ -253,14 +259,24 @@ WS_DLL_PUBLIC guint tvb_reported_length(const tvbuff_t *tvb);
 WS_DLL_PUBLIC gint tvb_reported_length_remaining(const tvbuff_t *tvb,
     const gint offset);
 
+/** Same as above, but throws an exception if the offset is out of bounds. */
+WS_DLL_PUBLIC guint tvb_ensure_reported_length_remaining(const tvbuff_t *tvb,
+    const gint offset);
+
 /** Set the reported length of a tvbuff to a given value; used for protocols
    whose headers contain an explicit length and where the calling
    dissector's payload may include padding as well as the packet for
    this protocol.
 
-   Also adjusts the data length. */
+   Also adjusts the available and contained length. */
 WS_DLL_PUBLIC void tvb_set_reported_length(tvbuff_t *tvb, const guint);
 
+/* Repair a tvbuff where the captured length is greater than the
+ * reported length; such a tvbuff makes no sense, as it's impossible
+ * to capture more data than is in the packet.
+ */
+WS_DLL_PUBLIC void tvb_fix_reported_length(tvbuff_t *tvb);
+
 WS_DLL_PUBLIC guint tvb_offset_from_real_beginning(const tvbuff_t *tvb);
 
 /* Returns the offset from the first byte of real data. */
@@ -409,6 +425,10 @@ WS_DLL_PUBLIC void tvb_get_letohguid(tvbuff_t *tvb, const gint offset,
 WS_DLL_PUBLIC void tvb_get_guid(tvbuff_t *tvb, const gint offset,
     e_guid_t *guid, const guint encoding);
 
+/* Fetches a byte array given a bit offset in a tvb */
+WS_DLL_PUBLIC guint8* tvb_get_bits_array(wmem_allocator_t *scope, tvbuff_t *tvb,
+    const gint offset, size_t length, size_t *data_length, const guint encoding);
+
 /* Fetch a specified number of bits from bit offset in a tvb.  All of these
  * functions are equivalent, except for the type of the return value.  Note
  * that the parameter encoding (where supplied) is meaningless and ignored */
@@ -537,10 +557,9 @@ WS_DLL_PUBLIC gint tvb_strnlen(tvbuff_t *tvb, const gint offset,
     const guint maxlength);
 
 /**
- * Format the data in the tvb from offset for size.  Returned string is
- * wmem packet_scoped so call must be in that scope.
+ * Format the data in the tvb from offset for size.
  */
-WS_DLL_PUBLIC gchar *tvb_format_text(tvbuff_t *tvb, const gint offset,
+WS_DLL_PUBLIC gchar *tvb_format_text(wmem_allocator_t *scope, tvbuff_t *tvb, const gint offset,
     const gint size);
 
 /**
@@ -552,10 +571,9 @@ WS_DLL_PUBLIC gchar *tvb_format_text_wsp(wmem_allocator_t* allocator, tvbuff_t *
 
 /**
  * Like "tvb_format_text()", but for null-padded strings; don't show
- * the null padding characters as "\000".  Returned string is wmem packet_scoped
- * so call must be in that scope.
+ * the null padding characters as "\000".
  */
-extern gchar *tvb_format_stringzpad(tvbuff_t *tvb, const gint offset,
+extern gchar *tvb_format_stringzpad(wmem_allocator_t *scope, tvbuff_t *tvb, const gint offset,
     const gint size);
 
 /**
@@ -591,7 +609,8 @@ WS_DLL_PUBLIC guint8 *tvb_get_string_enc(wmem_allocator_t *scope,
 /**
  * Given an allocator scope, a tvbuff, a bit offset, and a length in
  * 7-bit characters (not octets!), with the specified offset and
- * length referring to a string in the 3GPP TS 23.038 7bits encoding:
+ * length referring to a string in the 3GPP TS 23.038 7bits encoding,
+ * with code points packed into 7 bits:
  *
  *    allocate a buffer using the specified scope;
  *
@@ -608,11 +627,58 @@ WS_DLL_PUBLIC guint8 *tvb_get_string_enc(wmem_allocator_t *scope,
  * the memory allocated. Otherwise memory is automatically freed when the
  * scope lifetime is reached.
  */
-WS_DLL_PUBLIC gchar *tvb_get_ts_23_038_7bits_string(wmem_allocator_t *scope,
+WS_DLL_PUBLIC gchar *tvb_get_ts_23_038_7bits_string_packed(wmem_allocator_t *scope,
     tvbuff_t *tvb, const gint bit_offset, gint no_of_chars);
 
 /**
- * Given an allocator scope, a tvbuff, a bit offset, and a length in
+ * Given an allocator scope, a tvbuff, an offset, and a length in
+ * octets with the specified offset and length referring to a string
+ * in the 3GPP TS 23.038 7bits encoding, with one octet per code poiint
+ * (the 8th bit of each octet should be 0; if not, the octet is invalid):
+ *
+ *    allocate a buffer using the specified scope;
+ *
+ *    convert the string from the specified encoding to UTF-8, possibly
+ *    mapping some characters or invalid octet sequences to the Unicode
+ *    REPLACEMENT CHARACTER, and put the resulting UTF-8 string, plus a
+ *    trailing '\0', into that buffer;
+ *
+ *    and return a pointer to the buffer.
+ *
+ * Throws an exception if the tvbuff ends before the string does.
+ *
+ * If scope is set to NULL it is the user's responsibility to wmem_free()
+ * the memory allocated. Otherwise memory is automatically freed when the
+ * scope lifetime is reached.
+ */
+WS_DLL_PUBLIC gchar *tvb_get_ts_23_038_7bits_string_unpacked(wmem_allocator_t *scope,
+    tvbuff_t *tvb, const gint offset, gint length);
+
+/**
+ * Given an allocator scope, a tvbuff, an offset, and a length in
+ * octets with the specified offset and length referring to a string
+ * in the ETSI TS 102 221 Annex A encodings; if not:
+ *
+ *    allocate a buffer using the specified scope;
+ *
+ *    convert the string from the specified encoding to UTF-8, possibly
+ *    mapping some characters or invalid octet sequences to the Unicode
+ *    REPLACEMENT CHARACTER, and put the resulting UTF-8 string, plus a
+ *    trailing '\0', into that buffer;
+ *
+ *    and return a pointer to the buffer.
+ *
+ * Throws an exception if the tvbuff ends before the string does.
+ *
+ * If scope is set to NULL it is the user's responsibility to wmem_free()
+ * the memory allocated. Otherwise memory is automatically freed when the
+ * scope lifetime is reached.
+ */
+WS_DLL_PUBLIC gchar *tvb_get_etsi_ts_102_221_annex_a_string(wmem_allocator_t *scope,
+    tvbuff_t *tvb, const gint offset, gint length);
+
+/**
+ * Given an allocator scope, a tvbuff, an offset, and a length in
  * 7-bit characters (not octets!), with the specified offset and
  * length referring to a string in the ASCII 7bits encoding:
  *
@@ -762,9 +828,9 @@ WS_DLL_PUBLIC gboolean tvb_ascii_isprint(tvbuff_t *tvb, const gint offset,
  *  if "deseg" is false, return the amount of data remaining in
  *  the buffer.
  *
- * Set "*next_offset" to the offset of the character past the line
- * terminator, or past the end of the buffer if we don't find a line
- * terminator.  (It's not set if we return -1.)
+ * If "next_offset" is not NULL, set "*next_offset" to the offset of the
+ * character past the line terminator, or past the end of the buffer if
+ * we don't find a line terminator.  (It's not set if we return -1.)
  */
 WS_DLL_PUBLIC gint tvb_find_line_end(tvbuff_t *tvb, const gint offset, int len,
     gint *next_offset, const gboolean desegment);
@@ -783,9 +849,9 @@ WS_DLL_PUBLIC gint tvb_find_line_end(tvbuff_t *tvb, const gint offset, int len,
  * the end), or the amount of data remaining in the buffer if we don't
  * find a line terminator.
  *
- * Set "*next_offset" to the offset of the character past the line
- * terminator, or past the end of the buffer if we don't find a line
- * terminator.
+ * If "next_offset" is not NULL, set "*next_offset" to the offset of the
+ * character past the line terminator, or past the end of the buffer if
+ * we don't find a line terminator.
  */
 WS_DLL_PUBLIC gint tvb_find_line_end_unquoted(tvbuff_t *tvb, const gint offset,
     int len, gint *next_offset);
@@ -876,7 +942,7 @@ WS_DLL_PUBLIC gchar *tvb_bytes_to_str(wmem_allocator_t *allocator, tvbuff_t *tvb
  * tvbuff"), fetch BCD encoded digits from a tvbuff starting from either
  * the low or high half byte, formatting the digits according to an input digit
  * set, if NUL a default digit set of 0-9 returning "?" for overdecadic digits
- * will be used.  A pointer to the packet-scope (WMEM-allocated) string will
+ * will be used.  A pointer to the WMEM-allocated string will
  * be returned. Note a tvbuff content of 0xf is considered a 'filler' and will
  * end the conversion.
  */
@@ -886,10 +952,44 @@ typedef struct dgt_set_t
 }
 dgt_set_t;
 
-WS_DLL_PUBLIC const gchar *tvb_bcd_dig_to_wmem_packet_str(tvbuff_t *tvb,
-    const gint offset, const gint len, const dgt_set_t *dgt,
+WS_DLL_PUBLIC const gchar *tvb_bcd_dig_to_str(wmem_allocator_t *scope,
+    tvbuff_t *tvb, const gint offset, const gint len, const dgt_set_t *dgt,
     gboolean skip_first);
 
+/**
+ * Given a tvbuff, an offset into the tvbuff, and a length that starts
+ * at that offset (which may be -1 for "all the way to the end of the
+ * tvbuff"), fetch BCD encoded digits from a tvbuff starting from either
+ * the low or high half byte, formatting the digits according to an input digit
+ * set, if NUL a default digit set of 0-9 returning "?" for overdecadic digits
+ * will be used.  A pointer to the WMEM-allocated string will
+ * be returned. Note a tvbuff content of 0xf is considered a 'filler' and will
+ * end the conversion. Function uses big endian convetion: first digit is based
+ * on high order nibble, second digit is based on low order nibble.
+ */
+WS_DLL_PUBLIC const gchar *tvb_bcd_dig_to_str_be(wmem_allocator_t *scope,
+    tvbuff_t *tvb, const gint offset, const gint len, const dgt_set_t *dgt,
+    gboolean skip_first);
+
+/**
+ * Given a wmem scope, a tvbuff, an offset, a length, an input digit
+ * set, and a boolean indicator, fetch BCD-encoded digits from a
+ * tvbuff starting from either the low or high half byte of the
+ * first byte depending on the boolean indicator (TRUE means "start
+ * with the high half byte, ignoring the low half byte", and FALSE
+ * means "start with the low half byte and proceed to the high half
+ * byte), formating the digits into characters according to the
+ * input digit set, and return a pointer to a UTF-8 string, allocated
+ * using the wmem scope.  A high-order nibble of 0xf is considered a
+ * 'filler' and will end the conversion. If odd is set the high order
+ * nibble in the last octet will be skipped. If bigendian is set then
+ * high order nibble is taken as first digit of a byte and low order
+ * nibble as second digit.
+ */
+WS_DLL_PUBLIC gchar *tvb_get_bcd_string(wmem_allocator_t *scope, tvbuff_t *tvb,
+    const gint offset, gint len, const dgt_set_t *dgt,
+    gboolean skip_first, gboolean odd, gboolean bigendian);
+
 /** Locate a sub-tvbuff within another tvbuff, starting at position
  * 'haystack_offset'. Returns the index of the beginning of 'needle' within
  * 'haystack', or -1 if 'needle' is not found. The index is relative
@@ -1006,6 +1106,19 @@ WS_DLL_PUBLIC tvbuff_t *tvb_child_uncompress_lznt1(tvbuff_t *parent,
  */
 extern tvbuff_t* base64_to_tvb(tvbuff_t *parent, const char *base64);
 
+
+/** Return a tvb that contains the binary representation of a base64
+ *  encoded string in the parent tvb as a child of the indicated tvb.
+ *
+ * @param parent The parent tvbuff.
+ * @param offset Start of the base64 string in the tvb
+ * @param length Length of the base64 string in the tvb
+ *
+ * @return   A tvb with the binary representation of the base64 decoded string.
+ */
+extern tvbuff_t* base64_tvb_to_new_tvb(tvbuff_t* parent, int offset, int length);
+
+extern tvbuff_t* base64uri_tvb_to_new_tvb(tvbuff_t* parent, int offset, int length);
 /**
  * Extract a variable length integer from a tvbuff.
  * Each byte in a varint, except the last byte, has the most significant bit (msb)
